package com.massa.mrules.droolsfight.tests.drools;


import java.util.List;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatelessKnowledgeSession;
import org.junit.Test;

import com.massa.log.Log;
import com.massa.log.LogFactory;
import com.massa.mrules.droolsfight.beans.MeasureBean;
import com.massa.mrules.droolsfight.tests.AbstractTest;
import com.massa.mrules.droolsfight.tests.PrepareBeans;

public class DroolsTest extends AbstractTest {

	private static final Log LOG = LogFactory.getLog(DroolsTest.class);

	@Test
	public void test() throws Exception {

		final List<MeasureBean> beans = PrepareBeans.getAllBeans();

		LOG.info("RUNNING DROOLS DRL BEGIN");

		final long t0 = System.currentTimeMillis();
		final KnowledgeBase knowledgeBase = DroolsTest.createKnowledgeBaseFromDrl();
		final StatelessKnowledgeSession session = knowledgeBase.newStatelessKnowledgeSession();
		final long t1 = System.currentTimeMillis();

		int idx = 0;
		for (final MeasureBean bean: beans) {
			session.execute(bean);
			this.checkResult(LOG, bean, idx);
			++idx;
		}
		LOG.info("RUNNING DROOLS DRL END");

		LOG.info("RUNNING PERF DROOLS DRL BEGIN");
		final long t2 = System.currentTimeMillis();
		for (int i = 0 ; i < 10000 ; i++ ) {
			for (final MeasureBean bean: beans) {
				session.execute(bean);
			}
		}
		LOG.info("Time for compilation : " + (t1 - t0));
		LOG.info("Time for 10000 tests : " + (System.currentTimeMillis() - t2));
		LOG.info("RUNNING PERF DROOLS DRL END");
		LOG.info("");
	}

	private static KnowledgeBase createKnowledgeBaseFromDrl() throws Exception {
		final KnowledgeBuilder knowledgeBuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		knowledgeBuilder.add(ResourceFactory.newClassPathResource("drools_dt.drl"), ResourceType.DRL);

		final KnowledgeBaseConfiguration kconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
		//kconf.setOption(SequentialOption.YES);

		if (knowledgeBuilder.hasErrors()) {
			throw new RuntimeException("Errors: " + knowledgeBuilder.getErrors());
		}

		final KnowledgeBase knowledgeBase = KnowledgeBaseFactory.newKnowledgeBase(kconf);
		knowledgeBase.addKnowledgePackages(knowledgeBuilder.getKnowledgePackages());
		return knowledgeBase;
	}
}
