package com.massa.mrules.droolsfight.tests.java;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import com.massa.log.Log;
import com.massa.log.LogFactory;
import com.massa.mrules.droolsfight.beans.MeasureBean;
import com.massa.mrules.droolsfight.tests.AbstractTest;
import com.massa.mrules.droolsfight.tests.PrepareBeans;

public class JavaTest extends AbstractTest {

	private static final Log LOG = LogFactory.getLog(JavaTest.class);

	private static final List<IRule> RULES = new ArrayList<IRule>();
	static {
		JavaTest.RULES.add(new RuleDatesOk());
		JavaTest.RULES.add(new RuleLocationOk());
		JavaTest.RULES.add(new RuleContactsOk());
		JavaTest.RULES.add(new RuleSamplesOk());
		JavaTest.RULES.add(new RuleFinalize());
	}

	@Test
	public void test() {
		final List<MeasureBean> beans = PrepareBeans.getAllBeans();

		LOG.info("RUNNING JAVA BEGIN");
		int idx = 0;
		for (final MeasureBean bean: beans) {
			for (final IRule rule: JavaTest.RULES) {
				rule.test(bean);
			}

			this.checkResult(LOG, bean, idx);
			++idx;
		}
		LOG.info("RUNNING JAVA END");

		LOG.info("RUNNING PERF JAVA BEGIN");
		final long t0 = System.currentTimeMillis();
		for (int i = 0 ; i < 10000 ; i++ ) {
			for (final MeasureBean bean: beans) {
				bean.reset();
				for (final IRule rule: JavaTest.RULES) {
					rule.test(bean);
				}
			}
		}
		LOG.info("Time for 10000 tests : " + (System.currentTimeMillis() - t0));
		LOG.info("RUNNING PERF JAVA END");
		LOG.info("");
	}
}
