package com.massa.mrules.droolsfight.tests.mrules;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Test;

import com.massa.log.Log;
import com.massa.log.LogFactory;
import com.massa.mrules.CustomProperties;
import com.massa.mrules.builder.MRulesBuilder;
import com.massa.mrules.context.compile.CompilationLevel;
import com.massa.mrules.context.execute.MExecutionContext;
import com.massa.mrules.droolsfight.beans.MeasureBean;
import com.massa.mrules.droolsfight.tests.AbstractTest;
import com.massa.mrules.droolsfight.tests.PrepareBeans;
import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.set.IMruleExecutionSet;

public class MRulesTest extends AbstractTest {

	private static final Log LOG = LogFactory.getLog(MRulesTest.class);

	@Test
	public void testLight() throws Exception {
		this.test(CompilationLevel.LIGHT);
	}

	@Test
	public void testStd() throws Exception {
		this.test(CompilationLevel.STANDARD);
	}

	public void test(final CompilationLevel level) throws Exception {

		final List<MeasureBean> beans = PrepareBeans.getAllBeans();


		LOG.info("RUNNING MRULES BEGIN (" + level + ")");

		final long t0 = System.currentTimeMillis();
		final IMruleExecutionSet set = this.buildSet(level);
		final MExecutionContext<MeasureBean> ctxt = new MExecutionContext<MeasureBean>(set);
		final long t1 = System.currentTimeMillis();

		int idx = 0;
		for (final MeasureBean bean: beans) {
			ctxt.setInput(bean);
			ctxt.execute();

			this.checkResult(LOG, bean, idx);
			++idx;
		}

		LOG.info("RUNNING MRULES END");

		LOG.info("RUNNING PERF MRULES BEGIN (" + level + ")");
		final long t2 = System.currentTimeMillis();
		for (int i = 0 ; i < 10000 ; i++ ) {
			for (final MeasureBean bean: beans) {
				bean.reset();
				ctxt.setInput(bean);
				ctxt.execute();
			}
		}
		LOG.info("Time for compilation : " + (t1 - t0));
		LOG.info("Time for 10000 tests : " + (System.currentTimeMillis() - t2));
		LOG.info("RUNNING PERF MRULES END");
		LOG.info("");
	}

	public IMruleExecutionSet buildSet(final CompilationLevel level) throws MConfigurationException {
		MRulesBuilder.removeEngines("MEASURE_VALIDATION");

		final Map<String, String> parameters = new HashMap<String, String>();
		parameters.put(CustomProperties.ruleengine_uri.getLabel(), "MEASURE_VALIDATION");
		parameters.put(CustomProperties.configholder_compilation_level.getLabel(), level.name());
		parameters.put(CustomProperties.ruleenginebuilder_checkhash.getLabel(), "true");
		parameters.put(CustomProperties.configholder_impl.getLabel(), "com.massa.mrules.builder.XmlRuleEngineConfigHolder");
		parameters.put(CustomProperties.configholder_factoryimpl.getLabel(), "com.massa.mrules.factory.xml.RichXMLFactory");
		parameters.put(CustomProperties.xmlconfigholder_xmlfile.getLabel(), "mrules_dt.xml");

		final MRulesBuilder builder = new MRulesBuilder(parameters);
		return builder.getRuleExecutionSetInstance();
	}
}
